static char ident[] = "$Id: wlconfig.c,v 1.8 1998/12/05 19:34:15 stuart Exp $";

/* 
 * HISTORY
 * $Log: wlconfig.c,v $
 * Revision 1.8  1998/12/05 19:34:15  stuart
 * encryption support sent in by Steve Bellovin <smb@research.att.com>
 *
 */

#include <sys/param.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/types.h>

#include <net/if.h>
#include <net/if_types.h>
#include <net/if_dl.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>
#include <arpa/inet.h>

#define	NSIP
#include <netns/ns.h>
#include <netns/ns_if.h>
#include <netdb.h>

#define EON
#include <netiso/iso.h>
#include <netiso/iso_var.h>
#include <sys/protosw.h>

#include <ctype.h>
#include <err.h>
#include <errno.h>
#include <ifaddrs.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "if_wlreg.h"

static int beacon_get(const int s,
		      const char *if_name,
		      const char *beacon_index);
static int beacon_print(const beacon_cntr *b);
static int beacon_set(const int s,
		      const char cmd, 
		      const char *if_name, 
		      const char *eaddr);
static int dump_counters(const int s, const char *if_name);
static int dump_flags(const int flags);
static int gather_set(const int s, const char *if_name, const int gather);
static int parse_ether(u_char *out, const char *in);

#define numberof(a) (sizeof(a) / sizeof(a[0]))
main(argc, argv)
     int argc;
     char *argv[];
{
  int af = AF_INET;
  int s;
  char name[30];
  
  if (argc < 2) {
    fprintf(stderr, "usage: wlconfig interface\n%s%s%s%s%s%s%s",
	    "\t[ nwid [ id ] ]\n", 
	    "\t[ filter 0|1 ]\n", 
	    "\t[ add ether ]\n",
	    "\t[ del ether ]\n",
	    "\t[ get index ]\n",
	    "\t[ plain ]\n",
	    "\t[ key xx:xx:xx:xx:xx:xx:xx:xx ]\n");
    exit(1);
  }
  argc--, argv++;
  strncpy(name, *argv, sizeof(name));
  argc--, argv++;
  s = socket(af, SOCK_DGRAM, 0);
  if (s < 0) {
    perror("wlconfig: socket");
    exit(1);
  }
  if (!argc) {
    dump_counters(s, name);
    exit(0);
  }
  if (!strcmp("filter", *argv)) {
    u_long filt;
    struct ifnwid ifn;
    
    strncpy(ifn.ifn_name, name, sizeof(ifn.ifn_name));
    argc--; argv++;
    if (!argc) {
      strncpy(ifn.ifn_name, name, sizeof(ifn.ifn_name));

      if (ioctl(s, SIOCGIFNWID, (caddr_t)&ifn) < 0) {
	perror("ioctl (SIOCGIFNWID)");
	exit(1);
      }
      printf("%s: nwid=%02x%02x, filter is %s\n", 
	     ifn.ifn_name, 
	     ifn.nwid[0],
	     ifn.nwid[1],
	     ifn.nwid_mode ? "off" : "on");
    } else {
      ifn.cmd = SET_MODE;
      ifn.nwid_mode = strtoul(*argv, NULL, 0);
      if (ioctl(s, SIOCSIFNWID, (caddr_t)&ifn) < 0) {
	perror("ioctl (SIOCSIFNWID)");
	exit(1);
      }
    }
  }
  if (!strcmp("nwid", *argv)) {
    u_long nwid;
    struct ifnwid ifn;
    
    strncpy(ifn.ifn_name, name, sizeof(ifn.ifn_name));
    argc--; argv++;
    if (!argc) {
      strncpy(ifn.ifn_name, name, sizeof(ifn.ifn_name));

      if (ioctl(s, SIOCGIFNWID, (caddr_t)&ifn) < 0) {
	perror("ioctl (SIOCGIFNWID)");
	exit(1);
      }
      printf("%s: nwid=%02x%02x, filter is %s\n", 
	     ifn.ifn_name, 
	     ifn.nwid[0],
	     ifn.nwid[1],
	     ifn.nwid_mode ? "off" : "on");
    } else {
      ifn.cmd = SET_NWID;
      nwid = strtoul(*argv, NULL, 0);
      ifn.nwid[0] = (nwid / 256) & 0x0ff;
      ifn.nwid[1] = nwid & 0x0ff;
      if (ioctl(s, SIOCSIFNWID, (caddr_t)&ifn) < 0) {
	perror("ioctl (SIOCSIFNWID)");
	exit(1);
      }
    }
  }
  if (!strcmp("plain", *argv)) {
    struct ifencr ife;
    int i;

    strncpy(ife.ife_name, name, sizeof(ife.ife_name));
    ife.ife_encrypt = 0;
    for (i = 0; i < sizeof(ife.ife_key); i++)
	ife.ife_key[i] = 0;
    if (ioctl(s, SIOCSIFENCR, (caddr_t)&ife) < 0) {
      perror("ioctl (SIOCSIFENCR)");
      exit(1);
    }
  }
  if (!strcmp("key", *argv)) {
    struct ifencr ife;

    strncpy(ife.ife_name, name, sizeof(ife.ife_name));
    argc--; argv++;
    if (!argc) {
      if (ioctl(s, SIOCGIFENCR, (caddr_t)&ife) < 0) {
        perror("ioctl (SIOCGIFENCR)");
        exit(1);
      }
      printf("%s: %s %02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x\n", ife.ife_name,
  	ife.ife_encrypt ? "encrypted" : "plain",
  	ife.ife_key[0], ife.ife_key[1], ife.ife_key[2], ife.ife_key[3],
  	ife.ife_key[4], ife.ife_key[5], ife.ife_key[6], ife.ife_key[7]);
    }
    else {
      ife.ife_encrypt = 1;
      if (sscanf(argv[0], "%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x\n",
  	&ife.ife_key[0], &ife.ife_key[1], &ife.ife_key[2], &ife.ife_key[3],
  	&ife.ife_key[4], &ife.ife_key[5], &ife.ife_key[6], &ife.ife_key[7]) != sizeof ife.ife_key) {
	  fprintf(stderr, "wrong key format\n");
          exit(2);
      }
      if (ioctl(s, SIOCSIFENCR, (caddr_t)&ife) < 0) {
        perror("ioctl (SIOCSIFENCR)");
        exit(1);
      }
    }
  }
  if (!strcmp("flags", *argv)) {
    struct ifwflags ifw;

    argc--; argv++;
    strncpy(ifw.ifw_name, name, sizeof(ifw.ifw_name));
    if (ioctl(s, SIOCGIFWFLAGS, (caddr_t)&ifw) < 0) {
      perror("ioctl (SIOCGIFWFLAGS)");
      exit(1);
    }
    dump_flags(ifw.flags);
  }

  if (!strcmp("add", *argv) || !strcmp("del", *argv)) {
    char *cmd;
    char *eaddr;
    
    cmd = *argv;
    argc--; argv++;
    if (!argc) {
      fprintf(stderr, "missing ether addr for %s\n", cmd);
      exit(1);
    }
    eaddr = *argv;
    argc--; argv++;
    beacon_set(s, 
	       !strcmp("add", cmd) ? ADD_BEACON : DEL_BEACON, 
	       name, 
	       eaddr);
  }
  if (!strcmp("get", *argv)) {
    char *beacon_index;
    
    argc--; argv++;
    if (!argc) {
      beacon_index = NULL;
    } else {
      beacon_index = *argv;
    }
    argc--; argv++;
    beacon_get(s, name, beacon_index);
  }
  if (!strcmp("gather", *argv)) {
    argc--; argv++;
    gather_set(s, name, 1);
  }
  if (!strcmp("-gather", *argv)) {
    argc--; argv++;
    gather_set(s, name, 0);
  }
  exit(0);
}

static int beacon_get(const int s,
		      const char *if_name,
		      const char *beacon_index) {
  struct ifbeacon ifb;
  struct ifgather ifg;

  strncpy(ifb.ifb_name, if_name, sizeof(ifb.ifb_name));
  strncpy(ifg.ifg_name, if_name, sizeof(ifg.ifg_name));

  if (ioctl(s, SIOCGIFGATHER, (caddr_t)&ifg) < 0) {
    perror("ioctl (SIOCGIFGATHER)");
    exit(1);
  }
  printf("%s: gather is %s\n",
	 if_name, 
	 ifg.gather ? "enabled" : "disabled");
  if (beacon_index) {
    ifb.cmd = atoi(beacon_index);
    if (ioctl(s, SIOCGIFBEACON, (caddr_t)&ifb) < 0) {
      perror("ioctl (SIOCGIFBEACON)");
      exit(1);
    }
    printf("\tbeacons[%d]", ifb.cmd);
    beacon_print(&ifb.beacon);
  } else {
    for (ifb.cmd = 0; ; ifb.cmd++) {
      if (ioctl(s, SIOCGIFBEACON, (caddr_t)&ifb) < 0) {
	if (errno != EINVAL) {
	  perror("ioctl (SIOCGIFBEACON)");
	  exit(1);
	} else {
	  break;
	}
      }
      printf("\tbeacons[%d]", ifb.cmd);
      beacon_print(&ifb.beacon);
    }
  }
}

static int beacon_print(const beacon_cntr *b) {
  printf(" %02x:%02x:%02x:%02x:%02x:%02x %3u/%3u/%3u\n",
	 b->eth_addr[0],
	 b->eth_addr[1],
	 b->eth_addr[2],
	 b->eth_addr[3],
	 b->eth_addr[4],
	 b->eth_addr[5],
	 b->siglvl,
	 b->sillvl,
	 b->sigqual);
}

static int beacon_set(const int s,
		      const char cmd, 
		      const char *if_name, 
		      const char *eaddr) {
  struct ifbeacon ifb;
  
  bzero(&ifb, sizeof ifb);
  strncpy(ifb.ifb_name, if_name, sizeof(ifb.ifb_name));
  ifb.cmd = cmd;
  parse_ether(ifb.beacon.eth_addr, eaddr);
#if 0
  printf("%d %02x:%02x:%02x:%02x:%02x:%02x\n", 
	 cmd, 
	 ifb.beacon.eth_addr[0],
	 ifb.beacon.eth_addr[1],
	 ifb.beacon.eth_addr[2],
	 ifb.beacon.eth_addr[3],
	 ifb.beacon.eth_addr[4],
	 ifb.beacon.eth_addr[5]);
#endif
  if (ioctl(s, SIOCSIFBEACON, (caddr_t)&ifb) < 0) {
    perror("ioctl (SIOCSIFBEACON)");
    exit(1);
  }
}

/*
 * get and dump the interface counters
 */
static int dump_counters(const int s, const char *if_name) {
  int i;
  struct ifcntr ifc;
  
  strncpy(ifc.ifc_name, if_name, sizeof(ifc.ifc_name));
  if (ioctl(s, SIOCGIFCNTR, (caddr_t)&ifc) < 0) {
    perror("ioctl (SIOCGIFCNTR)");
    exit(1);
  }
  printf("%s:\n", ifc.ifc_name);
  printf("\txmt.xmt %u\n", ifc.counters.xmt.xmt);
  printf("\txmt.xmti %u\n", ifc.counters.xmt.xmti);
  printf("\txmt.defer %u\n", ifc.counters.xmt.defer);
  printf("\txmt.busy %u\n", ifc.counters.xmt.busy);
  printf("\txmt.sleaze %u\n", ifc.counters.xmt.sleaze);
  printf("\txmt.intrinsic %u\n", ifc.counters.xmt.intrinsic);
  printf("\txmt.intrinsic_count %u\n", ifc.counters.xmt.intrinsic_count);
  printf("\txmt.chain %u\n", ifc.counters.xmt.chain);
  printf("\txmt.heart %u\n", ifc.counters.xmt.heart);
  printf("\txmt.coll %u\n", ifc.counters.xmt.coll);
  printf("\txmt.ncoll %u\n", ifc.counters.xmt.ncoll);
  printf("\txmt.nodcd %u\n", ifc.counters.xmt.nodcd);
  printf("\txmt.nocts %u\n", ifc.counters.xmt.nocts);
  printf("\txmt.dma %u\n", ifc.counters.xmt.dma);
  printf("\trcv.rcv %u\n", ifc.counters.rcv.rcv);
  printf("\trcv.ovw %u\n", ifc.counters.rcv.ovw);
  printf("\trcv.crc %u\n", ifc.counters.rcv.crc);
  printf("\trcv.frame %u\n", ifc.counters.rcv.frame);
  printf("\trcv.rscerrs %u\n", ifc.counters.rcv.rscerrs);
  printf("\trcv.ovrnerrs %u\n", ifc.counters.rcv.ovrnerrs);
  printf("\trcv.partial %u\n", ifc.counters.rcv.partial);
  for (i = 0; i < 5; i++) {
    printf("\trcv.snr[%d] %u\n", i, ifc.counters.rcv.snr[i]);
  }
  printf("\tmmc.cur_nwid %02x%02x\n", 
	 ifc.counters.mmc.cur_nwid[0],
	 ifc.counters.mmc.cur_nwid[1]);
  printf("\tmmc.dce_status %u\n", ifc.counters.mmc.dce_status);
  printf("\tmmc.nwids %u\n", ifc.counters.mmc.nwids);
  printf("\tmmc.badnwids %u\n", ifc.counters.mmc.badnwids);
  printf("\tmmc.thrpreset %u\n", ifc.counters.mmc.thrpreset);
  printf("\tmmc.siglvl %u\n", ifc.counters.mmc.siglvl);
  printf("\tmmc.sillvl %u\n", ifc.counters.mmc.sillvl);
  printf("\tmmc.sigqual %u\n", ifc.counters.mmc.sigqual);
  printf("\tmmc.netwidl %u\n", ifc.counters.mmc.netwidl);
  printf("\tmmc.des %u\n", ifc.counters.mmc.des);
  printf("\tnwid %02x%02x\n", 
	 ifc.counters.nwid[0],
	 ifc.counters.nwid[1]);
  printf("\twatch %u\n", ifc.counters.watch);
  for (i = 0; i < WL_NBEACONS; i++) {
    printf("\tbeacons[%d]", i); 
    beacon_print(&ifc.counters.beacons[i]);
  }
}

static char *flag_names[] = {
  "RUNNING",
  "SEATED",
  "TBUSY",
  "MMI_INITTED",
  "CMD_WAIT",
  "CMD_REQUEST",
  "DEFERRED",
  "POWER_DOWN",
  "INIT_DEFERRED",
  "ALLNWID_MODE",
  "ATTACHED",
  "OVERRUNNING",
  "GATHER_SNR",
  "ALLMULTI",
  "V2CARD",
  "NOMODEM"
};

static int dump_flags(const int flags) {
  int i, j = 0;
  printf("flags %x: ", flags);
  for (i = 0; i < numberof(flag_names); i++) {
    if ((1 << i) & flags) {
      printf("%s%s", 
	     j ? ", " : "",
	     flag_names[i]);
      j++;
    }
  }
  printf("\n");
}

static int gather_set(const int s, const char *if_name, const int gather) {

  int i;
  struct ifgather ifg;

  strncpy(ifg.ifg_name, if_name, sizeof(ifg.ifg_name));
  ifg.gather = gather;
  if (ioctl(s, SIOCSIFGATHER, (caddr_t)&ifg) < 0) {
    perror("ioctl (SIOCSIFGATHER)");
    exit(1);
  }
}

/*
 * parse a "xx:xx:xx:xx:xx:xx"
 */
static int parse_ether(u_char *out, const char *in) {
  u_int temp[WAVELAN_ADDR_SIZE];
  int i;
  
  i = sscanf(in, 
	     "%x:%x:%x:%x:%x:%x",
	     &temp[0],
	     &temp[1],
	     &temp[2],
	     &temp[3],
	     &temp[4],
	     &temp[5]);
  if (i != WAVELAN_ADDR_SIZE) {
    fprintf(stderr, "can't figure out ether addr %s\n", in);
    exit(1);
  }
  for (i = 0; i < WAVELAN_ADDR_SIZE; i++) {
    out[i] = temp[i];
  }
}
